namespace StudySystemClient {
	public class ActivitiesView : Gtk.Box {
		private Client client;
		private RefreshingIndicator refreshing_indicator;
		private CardArea card_area;

		public ActivitiesView(Client client)  {
			this.client = client;
			card_area = new CardArea();

			var overlay = new Gtk.Overlay();
			overlay.hexpand = overlay.vexpand = true;
			overlay.set_child(card_area);
			this.append(overlay);

			refreshing_indicator = new RefreshingIndicator(overlay);

			this.map.connect(refresh);
		}

		private async void refresh() {
			refreshing_indicator.show();
			try {
				var activities = yield client.list_activities();
				card_area.clear();
				foreach (var activity in activities) {
					var card = new ActivityCard(activity);
					card.session_logged.connect(log_session);
					card_area.add(card);
				}
			} catch (ClientError e) {
				stderr.printf("Error refreshing activities: %s\n",
							  e.message);
			}
			refreshing_indicator.hide();
		}

		private async void log_session(string subject, ActivityType type,
									   int minutes) {
			try {
				yield client.log_session(subject, type, minutes);
				yield refresh();
			} catch (ClientError e) {
				stderr.printf("Error logging session: %s\n", e.message);
			}
		}
	}

	private class RefreshingIndicator {
		private Gtk.Overlay overlay;
		private Gtk.Frame frame;

		public RefreshingIndicator(Gtk.Overlay overlay) {
			this.overlay = overlay;

			var label = new Gtk.Label("Refreshing");
			label.halign = Gtk.Align.START;

			var spinner = new Gtk.Spinner();
			spinner.start();

			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);
			content.margin_top = content.margin_bottom
				= content.margin_start = content.margin_end = 12;
			content.append(label);
			content.append(spinner);

			frame = new Gtk.Frame(null);
			frame.halign = Gtk.Align.CENTER;
			frame.valign = Gtk.Align.START;
			frame.add_css_class("osd");
			frame.add_css_class("popdown");
			frame.set_child(content);

			overlay.add_overlay(frame);
		}

		public void show() {
			frame.add_css_class("visible");
		}

		public void hide() {
			frame.remove_css_class("visible");
		}
	}
}
